from typing import Dict, List, Union, TypeVar

K = TypeVar('K')
V = TypeVar('V')


def get(d: Union[Dict[K, V], Dict[K, Dict]], k: Union[List[K], K]) -> Union[None, V]:
    """
    Get the value of a key or nested keys in a dictionary, returning None if it doesn't exist.
    :param d: dictionary to look in
    :param k: key or list of keys
    :return: an item in the dictionary, or None
    """
    if not isinstance(k, list):
        return d.get(k)

    if len(k) == 0:
        return None

    res = d
    for key in k:
        if res is None:
            return None
        res = res.get(key)

    return res



from jettools import dict
import unittest


class TestGetNestedKey(unittest.TestCase):
    def test_should_return_none_if_no_keys(self):
        d = {'level1': {'level1': 2}}
        self.assertEqual(dict.get(d, []), None)

    def test_should_grab_val_if_just_string(self):
        d = {'level1': 3}
        self.assertEqual(dict.get(d, 'level1'), 3)

    def test_should_get_nested_value_if_exists(self):
        d = {'level1': {'level2': 2}}
        self.assertEqual(dict.get(d, ['level1', 'level2']), 2)

    def test_should_return_none_if_fake_levels(self):
        d = {'level1': {'level2': 2}}
        self.assertEqual(dict.get(d, ['level2', 'level3']), None)



from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

setup(
    name='jetts-tools',
    version='0.0.2',
    author='Jett Crowson',
    author_email='jettcrowson@gmail.com',
    url='https://github.com/jettdc/jetts-tools',
    description='Various useful python utility functions.',
    long_description=readme,
    long_description_content_type='text/markdown',
    license='MIT',
    packages=find_packages('jettools'),
    keywords='python tools utils',
    python_requires='>=3.5'
)
